/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j;

import org.opt4j.core.start.Opt4JTask;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provider;

/**
 * Specialized {@link Opt4JTask} that allows access to the {@link Injector} used in Opt4J to
 * implement the dependency-driven decoding.
 * This {@link Opt4JTask} implements a provider of the opt4J {@link Injector} in order to make it
 * accessible for decoders and evaluators. An improvement to this generic {@link Injector} retrieval
 * limited to Guice (Abstract)Modules that cannot be used here (no polymorphism in Java).
 * 
 * @author diewald
 */
public class Opt4JDseTask extends Opt4JTask implements Provider<Injector> {

	/** No-arg constructor. */
	@Inject
	public Opt4JDseTask() {
		super(true);
	}

	/**
	 * Constructs a {@link Opt4JTask}.
	 * 
	 * @param closeOnStop
	 *            close automatically after optimization
	 */
	public Opt4JDseTask(boolean closeOnStop) {
		super(closeOnStop);
	}

	/** {@inheritDoc} */
	@Override
	public Injector get() {
		return injector;
	}
}
