/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j;

import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.CompositeExplorationSolution;
import org.fortiss.af3.exploration.alg.dse.SolutionQuantification;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ProblemModuleBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.IExplorationSolution;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.Individual;
import org.opt4j.core.Objective;
import org.opt4j.core.optimizer.Archive;

/**
 * Result of an Opt4J run, i.e. the {@link SolutionQuantification}s for all pareto-optimal
 * {@link IExplorationSolution}s.
 * 
 * @author barner
 */
public class Opt4JExplorationSolutionSet<P extends Phenotype> extends
		CompositeExplorationSolution<P> {

	/**
	 * Constructs an {@link CompositeExplorationSolution} representation from the results obtained
	 * by an Opt4J
	 * run, i.e. the {@link SolutionQuantification}s for all pareto-optimal
	 * {@link IExplorationSolution}s.
	 * 
	 * @throws Exception
	 *             if the evaluation result for an {@link Objective} of an individual cannot be
	 *             identified.
	 */
	Opt4JExplorationSolutionSet(Archive archive,
			Map<ExplorationTarget<?>, ExplorationTarget<?>> copiedTargetAssocMap,
			ProblemModuleBase<?, ?, ?> problemModule) throws Exception {
		for(Individual individual : archive) {
			@SuppressWarnings("unchecked") IExplorationSolution<P> expSolution =
					(IExplorationSolution<P>)individual.getPhenotype();
			addExplorationSolution(expSolution,
					new Opt4JSingleExplorationSolution(individual.getObjectives(),
							copiedTargetAssocMap, problemModule));
		}
	}
}
