/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FaultDetectionVotingTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FaultDetectionVotingTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.FlatGenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;

import com.google.inject.Inject;

/**
 * Creator of random task mappings for fail-silent execution model
 * 
 * @author diewald
 */
public class FaultDetectionVotingTaskMappingCreator
		extends
		TaskMappingCreator<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding> {

	/**
	 * Constructs a task mapping creator for a given DSE specification and input model
	 * (execution model = fault-detection and voting)
	 */
	@Inject
	public FaultDetectionVotingTaskMappingCreator(DecoderDependencyGraph execDepGraph) {
		super(execDepGraph);
	}

	/** {@inheritDoc} */
	@Override
	public FaultDetectionVotingTaskMappingEncoding createComposable(
			FlatGenotypeMap createdGenotypes, FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		AbstractTaskGraphEncoding atgEnc =
				decodedPhenotypes.getEncoding(AbstractTaskGraphEncoding.class);
		assert (atgEnc != null) : "The required abstract task graph encoding could not be found in the map of already decoded phenotypes.";
		AbstractTaskMappingEncoding atmEnc =
				createdGenotypes.get(AbstractTaskMappingEncoding.class);
		assert (atmEnc != null) : "The required abstract task mapping encoding could not be found in the map of already decoded phenotypes.";
		PartitionMappingEncoding partEnc = createdGenotypes.get(PartitionMappingEncoding.class);
		assert (partEnc != null) : "The required partition mapping encoding could not be found in the map of already decoded phenotypes.";
		return new FaultDetectionVotingTaskMappingEncoding(partEnc, atgEnc, atmEnc);
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends IExplorationEncoding> getCreatedEncodingType() {
		return InstantiatedTaskMappingEncoding.class;
	}
}
