/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem;

import org.fortiss.af3.exploration.alg.dse.TaskMappingFactory;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.comm.MessageEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.InstantiatedAcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.dse.module.DecoderModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.FailSilentExecModelFactory;
import org.fortiss.af3.exploration.alg.dse.sysmodel.sched.comm.TDMABusScheduler;
import org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc.EDFSchedulerBase;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;

/**
 * The Decoder decodes {@link Genotype}s into phenotype objects.
 * 
 * @author huang
 */
public class StrictTTDecoder<S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
		extends DecoderModule<StrictTTSchedule<S, T>> {
	/** Underlying EDF task scheduler */
	private EDFSchedulerBase<S, T> simpleEDFScheduler;

	/** Underlying bus scheduler */
	private TDMABusScheduler<S, T> simpleBusScheduler;

	/**
	 * Constructs a new decoder that constructs {@link StrictTTSchedule}s for a given exploration
	 * run.
	 */
	@SuppressWarnings("unchecked")
	public StrictTTDecoder() {
		// FIXME: HACK to create the FailSilentTaskMappingFactory. Support for FaultDetection will
		// be added again after switching to DI mechanisms completely.
		// TaskMappingFactory<?, ?> tmFactory = TaskMappingFactory.getInstance();
		TaskMappingFactory<?, ?> tmFactory = FailSilentExecModelFactory.getInstance();
		simpleEDFScheduler = (EDFSchedulerBase<S, T>)tmFactory.createEDFScheduler();
		simpleBusScheduler = new TDMABusScheduler<S, T>();
	}

	@Provides
	public StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding> decode(
			FailSilentTaskMappingEncoding itmEnc,
			MessageEncoding<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding> msgEnc,
			InstantiatedAcyclicTaskGraphEncoding<FailSilentTaskMappingEntry> acycItgEnc,
			PlatformCommunicationGraphEncoding pcgEncoding, PartitionMappingEncoding partEnc) {
		return decode(itmEnc, msgEnc, acycItgEnc, pcgEncoding, partEnc);
	}

	public StrictTTSchedule<S, T> decode(T itmEnc, MessageEncoding<S, T> msgEnc,
			InstantiatedAcyclicTaskGraphEncoding<S> acycItgEnc,
			PlatformCommunicationGraphEncoding pcgEncoding, PartitionMappingEncoding partEnc) {
		// TODO: Externalize the validation such that the encoding is not required to be passed to
		// the constructor (is there for validation purposes)
		StrictTTSchedule<S, T> schedule = new StrictTTSchedule<S, T>(itmEnc);
		simpleEDFScheduler.schedule(acycItgEnc, partEnc, itmEnc, schedule);
		assert (schedule.validateSchedule(true));

		// TODO: Find out why the message scheduler does not require the task graph. Actually, it
		// should require it...
		simpleBusScheduler
				.schedule(schedule, msgEnc.getPeriodicMessages(), acycItgEnc, pcgEncoding);
		assert (schedule.validateSchedule(true));

		return schedule;
	}

	/** {@inheritDoc} */
	@Override
	public StrictTTSchedule<S, T> decode(Genotype genotype) {
		return null;
	}
}
