/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.exception;

import org.fortiss.af3.exploration.model.ExplorationConstraint;

/**
 * Exception for failures in the automatic constraint generation. Displays some information about
 * the failed constraint.
 * 
 * @author diewald
 */
public class ConstraintGenerationException extends ExplorationException {

	/**
	 * Constructs a {@link ConstraintGenerationException} with information about the failed
	 * {@link ExplorationConstraint} and the given message.
	 */
	public ConstraintGenerationException(Class<? extends ExplorationConstraint<?>> constraintType,
			String msg) {
		super("Failed to generate a " + constraintType.getSimpleName() + ": " + msg);
	}

	/**
	 * Constructs a {@link ConstraintGenerationException} with information about the failed
	 * {@link ExplorationConstraint} and the given message and the given {@link Exception}.
	 */
	public ConstraintGenerationException(Class<? extends ExplorationConstraint<?>> constraintType,
			String msg, Exception e) {
		super("Failed to generate a " + constraintType.getSimpleName() + ": " + msg, e);
	}
}
